package cn.ezhizao.project.business.controller;

import cn.ezhizao.common.constant.HttpStatus;
import cn.ezhizao.common.utils.SecurityUtils;
import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.domain.*;
import cn.ezhizao.project.business.service.*;
import cn.ezhizao.project.system.domain.SysUser;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageInfo;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 报工Controller
 *
 * @author ezhizao
 * @date 2023-11-29
 */
@RestController
@RequestMapping("/business/productionDaywork")
public class BizDayworkController extends BaseController {
    @Resource
    private IBizDayworkService bizDayworkService;

    @Resource
    private IBizTechnologicalProcessDetailService bizTechnologicalProcessDetailService;

    @Resource
    private IBizLotTechnologicalProcessDetailService bizTechnologicalAmendDetailService;

    @Resource
    private IBizOutsourcedOrderDetailService bizOutsourcedOrderDetailService;

    @Resource
    private IBizDayworkCarrierService bizDayworkCarrierService;

    @Resource
    private IBizLotTechnologicalProcessDetailService bizLotTechnologicalProcessDetailService;

    @Resource
    private IBizLotService bizLotService;

    @Resource
    private IBizProcessService bizProcessService;
    @Resource
    private IBizDayworkQualifiedService bizDayworkQualifiedService;
    @Resource
    private IBizDayworkItemService bizDayworkItemService;
    @Resource
    private IBizDayworkItemRejectService bizDayworkItemRejectService;

    @Resource
    private IBizReturnReceiptDetailService bizReturnReceiptDetailService;
    @Resource
    private IBizProductionResourceGroupDetailService bizProductionResourceGroupDetailService;
    @Resource
    private IBizLotTechnologicalProcessService bizLotTechnologicalProcessService;
    @Resource
    private IBizProductionPlanDetailService bizProductionPlanDetailService;

    @Resource
    HttpServletRequest request;

    /**
     * 查询报工列表
     */
    @PreAuthorize("@ss.hasPermi('business:daywork:list')")
    @GetMapping("/list")
    public TableDataInfo list(BizDaywork bizDaywork) {
        startPage();

        //查询当前生产子计划的批次报工列表
        List<BizDaywork> dayworkList = bizDayworkService.getList(bizDaywork);
        for (BizDaywork daywork : dayworkList) {
            if (daywork.getTotalWorkingHours() == null) {
                daywork.setTotalWorkingHours(Long.valueOf(0));
            }
        }
        //最终返回当前子计划中批次报工的工序是这个人员负责的工序
        return getDataTable(dayworkList);
    }

    @GetMapping("/listDayworkLot")
    public TableDataInfo listDayworkLot(BizDaywork bizDaywork) {
        startPage();
        List<BizDaywork> dayworkList = bizDayworkService.getListFromNoCurrent(bizDaywork);
        if (dayworkList.size() == 0) {
            return getDataTable(dayworkList);
        }
        List<BizProductionPlanDetail> productionPlanDetail = bizProductionPlanDetailService.query().in("id", dayworkList.stream().map(BizDaywork::getProductionPlanDetailId).collect(Collectors.toList())).list();
        productionPlanDetail.forEach(productionPlan -> {
            dayworkList.forEach(daywork -> {
                if (productionPlan.getId().equals(daywork.getProductionPlanDetailId())) {
                    daywork.setProductDescription(productionPlan.getProductDescription());
                }
            });
        });

        return getDataTable(dayworkList);
    }

    @GetMapping("/listForOutsource")
    public TableDataInfo listForOutsource(BizDaywork bizDaywork) {
        // 已经在页面中存在的daywork id
        List<Long> existingDayworkIds = bizDaywork.getExistingDayworkIds();
        BizDayworkItem dayworkItem = new BizDayworkItem();
        dayworkItem.setDeptCode(bizDaywork.getDeptCode());
        List<BizDayworkItem> dayworkItemList = bizDayworkService.getItemListForOutsource(dayworkItem);

        List<Long> dayworkIds = dayworkItemList.stream().map(BizDayworkItem::getDayworkId).collect(Collectors.toList());
        // 如果得到的集合是空集合，放入一个0元素，传到xml中，进行SQL组合
        // 否则，SQL组合，不会进行 IN 的判断，导致把所有数据都得到了。
        if (dayworkIds.isEmpty()) {
            dayworkIds.add(0L);
        }
        bizDaywork.setDayworkIds(dayworkIds);
        startPage();
        List<BizDaywork> dayworkList = bizDayworkService.getListForOutsource(bizDaywork);
        long total = new PageInfo(dayworkList).getTotal();
        //根据查询的daywork信息上发出单查询是否存在
        List<BizOutsourcedOrderDetail> orderDetails = bizOutsourcedOrderDetailService.query().in("daywork_id", dayworkIds).eq("deleted", 0).list();
        //如果有查询是否回收
        if (orderDetails.size() > 0) {
            //以回收的数据
            List<BizReturnReceiptDetail> returnReceiptDetails = bizReturnReceiptDetailService.query().in("outsource_detail_id", orderDetails.stream().map(BizOutsourcedOrderDetail::getId).collect(Collectors.toList())).eq("deleted", 0).list();

            //如果数据相同则表示全部回收
            if (orderDetails.size() != returnReceiptDetails.size()) {
                // 将 returnReceiptDetails 中的 id 收集到一个 Set 中，以便快速查找
                Set<Long> returnReceiptDetailIds = returnReceiptDetails.stream()
                        .map(BizReturnReceiptDetail::getOutsourceDetailId) // 假设 getId() 返回 BizReturnReceiptDetail 对象的 id
                        .collect(Collectors.toSet());

                // 使用 stream() 和 filter() 筛选出那些 id 不在 returnReceiptDetailIds 中的 orderDetails
                List<BizOutsourcedOrderDetail> filteredOrderDetails = orderDetails.stream()
                        .filter(orderDetail -> !returnReceiptDetailIds.contains(orderDetail.getId())) // 筛选出 id 不在 returnReceiptDetails 中的元素
                        .collect(Collectors.toList());

                // 使用 Stream API 收集所有唯一的 dayworkId 到一个 Set 中
                Set<Long> uniqueDayworkIds = filteredOrderDetails.stream()
                        .map(BizOutsourcedOrderDetail::getDayworkId) // 假设 getDayworkId() 返回 BizOutsourcedOrderDetail 对象的 dayworkId
                        .collect(Collectors.toSet());

                //将未回收的从集合中移除
                // 使用 stream() 和 filter() 创建一个新的 dayworkList，排除那些 dayworkId 出现在 dayworkIdsToDelete 中的 BizDaywork 对象
                List<BizDaywork> updatedDayworkList = dayworkList.stream()
                        .filter(daywork -> !uniqueDayworkIds.contains(daywork.getId())) // 假设 getId() 返回 BizDaywork 对象的 id
                        .collect(Collectors.toList());

                dayworkList = updatedDayworkList;
            }
        }

        dayworkList.forEach(daywork -> {
            dayworkItemList.forEach(item -> {
                if (daywork.getId().equals(item.getDayworkId())) {
                    daywork.setProductNum(item.getQualifiedNum());
                }
            });

        });
        TableDataInfo dataTable = getDataTable(dayworkList);
        dataTable.setTotal(total);

        return dataTable;
    }

    @GetMapping("/processesForOutsource")
    public TableDataInfo listProcessesForOutsource(BizDaywork bizDaywork) {
        //查找当前生产子计划的批次报工信息，找到最后一条包报工信息
        BizDaywork daywork = bizDayworkService.query().eq("lot_id", bizDaywork.getLotId()).eq("deleted", 0).one();
        BizDayworkItem dayworkItem = new BizDayworkItem();
        dayworkItem.setDayworkId(daywork.getId());
        //List<BizDayworkItem> dayworkItemList = bizDayworkItemService.getList(dayworkItem);
        //用户再次提交时传入外协明细id
        dayworkItem.setOutsourceDetailId(bizDaywork.getOutsourceDetailId());
        List<BizDayworkItem> dayworkItemList = bizDayworkItemService.getOutsourcesItemList(dayworkItem);
        List<BizDayworkItem> finishedItem = bizDayworkItemService.getLastFinished(dayworkItem);
        if (dayworkItemList.size() > 0) {
            dayworkItem = dayworkItemList.get(0);
        } else {
            List<BizTechnologicalProcessDetail> processList = new ArrayList<>();
            return getDataTable(processList);
        }
        dayworkItem.setKeyword(bizDaywork.getKeyword());

        //根据当前工序查找之后的工序列表
        //是否单批单改或废品回用
//        startPage();
        if (daywork.getIsAmend() == 1 || daywork.getIsWasteRecycling() == 1) {
            List<BizLotTechnologicalProcessDetail> processList = bizLotTechnologicalProcessDetailService.getProcessList(dayworkItem).stream().filter(v -> v.getProcessStepNumber().compareTo(finishedItem.get(0).getProcessStepNumber()) > 0).collect(Collectors.toList());
            return getDataTable(processList);
        } else {
            List<BizTechnologicalProcessDetail> processList = bizTechnologicalProcessDetailService.getProcessList(dayworkItem).stream().filter(v -> v.getProcessStepNumber().compareTo(finishedItem.get(0).getProcessStepNumber()) > 0).collect(Collectors.toList());
            return getDataTable(processList);
        }

    }

    @GetMapping("/getLotPosition")
    public TableDataInfo getLotPosition(BizDaywork bizDaywork) {
        startPage();
        List<BizDaywork> list = bizDayworkService.getLotPosition(bizDaywork);
        list.stream().forEach(item -> {
            if (item.getTurnoverArea().equals("车间内周转看place字段")) {
                item.setTurnoverArea(null);
            }
        });
        return getDataTable(list);
    }

    /**
     * 查询报工列表
     *
     * @param bizDayworkItem 查询条件
     * @return TableDataInfo
     */
    @GetMapping("/listItem")
    public TableDataInfo listItem(BizDayworkItem bizDayworkItem) throws NoSuchFieldException, IllegalAccessException {
        startPage();
        //查询批次报工信息列表
        List<BizDayworkItem> itemList = bizDayworkService.getItemList(bizDayworkItem);

        List<BizLotTechnologicalProcessDetail> lotProcessList = new ArrayList<>();//新工序列表
        List<BizTechnologicalProcessDetail> processList = new ArrayList<>();//标准工序列表
        if (!itemList.isEmpty()) {
            //是否废回用或单批但改
            if (bizDayworkItem.getIsAmend() == 1 || bizDayworkItem.getIsWasteRecycling() == 1) {
                //查询新工序列表
                lotProcessList = bizLotTechnologicalProcessDetailService.query().eq("lot_technological_process_id", itemList.get(0).getTechnologicalProcessId()).list();
            } else {
                //查询标准工列表
                processList = bizTechnologicalProcessDetailService.query().eq("technological_process_id", itemList.get(0).getTechnologicalProcessId()).list();
            }
        }
        for (BizDayworkItem item : itemList) {
            if (item.getRejectNum() == null) {
                item.setRejectNum(0);
            }
            //是单批单改或废品回用
            //根据当前工序id设置当前工序
            if (bizDayworkItem.getIsAmend() == 1 || bizDayworkItem.getIsWasteRecycling() == 1) {
                lotProcessList.forEach(lotProcess -> {
                    if (item.getTechnologicalProcessDetailId().equals(lotProcess.getId())) {
                        item.setProcessAlias(lotProcess.getProcessAlias());
                    }
                });
            } else {
                processList.forEach(process -> {
                    if (item.getTechnologicalProcessDetailId().equals(process.getId())) {
                        item.setProcessAlias(process.getProcessAlias());
                    }
                });
            }
        }
        //如果进行到最后一条报工，则不能删除，拿到工艺计划单的最后一道工序步骤编码
        String lastProcessStepNumber;
        String firstProcessStepNumber;
        if (bizDayworkItem.getIsAmend() == 1 || bizDayworkItem.getIsWasteRecycling() == 1) {
            lastProcessStepNumber = bizLotTechnologicalProcessDetailService.query().eq("lot_technological_process_id", bizDayworkItem.getTechnologicalProcessId())
                    .orderByDesc("process_step_number").list().get(0).getProcessStepNumber();
            firstProcessStepNumber = bizLotTechnologicalProcessDetailService.query()
                    .ne("process_key", "BL")
                    .eq("lot_technological_process_id", bizDayworkItem.getTechnologicalProcessId())
                    .orderByAsc("process_step_number").list().get(0).getProcessStepNumber();
        } else {
            lastProcessStepNumber = bizTechnologicalProcessDetailService.query()
                    .eq("technological_process_id", bizDayworkItem.getTechnologicalProcessId())
                    .ne("tenant_id", 9).orderByDesc("process_step_number").list().get(0).getProcessStepNumber();
            firstProcessStepNumber = bizTechnologicalProcessDetailService.query()
                    .ne("process_key", "BL")
                    .eq("technological_process_id", bizDayworkItem.getTechnologicalProcessId())
                    .ne("tenant_id", 9).orderByAsc("process_step_number").list().get(0).getProcessStepNumber();
        }
        TableDataInfo info = getDataTable(itemList);
        Map<String, Object> map = new HashMap<>();

        map.put("latestProcessId", bizDayworkItemService.getLatestDayworkItemId(bizDayworkItem));
        map.put("lastItem", bizDayworkItemService.getLastItem(bizDayworkItem));
        map.put("lastProcessStepNumber", lastProcessStepNumber);
        map.put("isFirst", firstProcessStepNumber);
        info.setOthers(map);
        return info;
    }

    /**
     * 查询报工日报列表
     *
     * @param bizDayworkItem 查询条件
     * @return TableDataInfo
     */
    @GetMapping("/listReportItem")
    public TableDataInfo listReportItem(BizDayworkItem bizDayworkItem) throws NoSuchFieldException, IllegalAccessException {
        startPage();
        //查询批次报工信息列表
        List<BizDayworkItem> itemList = bizDayworkService.listReportItem(bizDayworkItem);
        List<Long> lotIds = itemList.stream().filter(item -> item.getProdNum() == 0).map(BizDayworkItem::getLotId).collect(Collectors.toList());
        List<BizLot> lotList = bizLotService.query().in("id", lotIds.isEmpty() ? Collections.singletonList(0L) : itemList.stream().filter(item -> item.getProdNum() == 0).map(BizDayworkItem::getLotId).collect(Collectors.toList())).list();
        //首序投产数为0，所以查询如果prodNum为0，其他是第一条报工的时候，给prodNum赋值单批量
        itemList.forEach(item -> {
            if (item.getProdNum() == 0) {
                //拿到第一条报工的工序步骤，为了判断这个投产数为0，是不是该工序步骤的报工，如果是，就给它的投产量赋值为单批量
                String processStepNumber = itemList.stream().filter(v -> v.getDayworkId().equals(item.getDayworkId())).sorted(Comparator.comparing(BizDayworkItem::getProcessStepNumber)).collect(Collectors.toList()).get(0).getProcessStepNumber();
                if (item.getProcessStepNumber().equals(processStepNumber)) {
                    item.setProdNum(lotList.stream().filter(lot -> lot.getId().equals(item.getLotId())).findFirst().get().getProductionQuantity());
                }
            }
            if (item.getRejectNum() == null) {
                item.setRejectNum(0);
            }
        });

        return getDataTable(itemList);
    }

    /**
     * 修改报工列表
     *
     * @param bizDayworkItem
     * @return TableDataInfo
     */
    @PostMapping("/updateItem")
    public AjaxResult updateItem(@RequestBody BizDayworkItem bizDayworkItem) {
//        String tenantId = request.getHeader("tenantId");
//        bizDayworkItem.setTenantId(Long.valueOf(tenantId));
        return toAjax(bizDayworkItemService.saveOrUpdate(bizDayworkItem));
    }

    /**
     * 获取工序列表
     */
    @GetMapping("/getProcessList")
    public AjaxResult getProcessList(BizDayworkItem bizDayworkItem) {
        if (bizDayworkItem.getIsAmend() == 1 || bizDayworkItem.getIsWasteRecycling() == 1) {
//            BizLotTechnologicalProcess bizLotTechnologicalProcess = bizLotTechnologicalProcessService.query().eq("lot_id", bizDayworkItem.getLotId()).eq("is_stop",0).list().get(0);
//            List<BizLotTechnologicalProcessDetail> bizLotTechnologicalProcessDetail = bizLotTechnologicalProcessDetailService.query().eq("lot_technological_process_id", bizLotTechnologicalProcess.getId()).list();
            List<BizLotTechnologicalProcessDetail> bizLotTechnologicalProcessDetail = bizLotTechnologicalProcessDetailService.query().eq("lot_technological_process_id", bizDayworkItem.getTechnologicalProcessId()).list();
            List<BizProcess> process = new ArrayList<>();
            for (BizLotTechnologicalProcessDetail item : bizLotTechnologicalProcessDetail) {
                BizProcess bizProcess = new BizProcess();
                bizProcess.setLabel(item.getProcessAlias());
                bizProcess.setValue(item.getProcessId());
                process.add(bizProcess);
            }
            return success(process);
        }
        return success(bizDayworkService.getProcessList(bizDayworkItem));
    }

    //删除报工明细
    @DeleteMapping("/{ids}")
    public AjaxResult deletedItem(@PathVariable Long ids) {
        //判断如果是首序，则必须留一条报工
        BizDayworkItem bizDayworkItem = bizDayworkItemService.getById(ids);
        BizDayworkItem item = new BizDayworkItem();
        item.setDayworkId(bizDayworkItem.getDayworkId());
        //第一条报工
        BizDayworkItem firstItem = bizDayworkItemService.getList(item).stream().sorted(Comparator.comparing(BizDayworkItem::getCreateTime)).collect(Collectors.toList()).get(0);
        item.setProcessStepNumber(bizDayworkItem.getProcessStepNumber());
        //拿到同一个工序步骤的报工信息
        List<BizDayworkItem> totalItem = bizDayworkItemService.getList(item);
        //如果要删除的报工是这个工序的最后一条报工，且是首序
        if (totalItem.size() == 1 && bizDayworkItem.getProcessStepNumber().equals(firstItem.getProcessStepNumber())) {
            return error("首序报工至少存在一条");
        } else {
            bizDayworkItemService.removeById(bizDayworkItem);
        }

//        List<Long> id = new ArrayList<>();
//        id.add(bizDayworkItem.getDayworkId());
//        List<BizDayworkItem> itemList = bizDayworkItemService.getListByDayworkIds(id).stream().filter(item -> item.getProcessStepNumber().equals(bizDayworkItem.getProcessStepNumber())).collect(Collectors.toList());
////        List<BizDayworkItem> turnoverList = itemList.stream().filter(item -> "4".equals(item.getStatus()) || "5".equals(item.getStatus()) || "7".equals(item.getStatus())).collect(Collectors.toList());
//        BizDaywork daywork = bizDayworkService.query().eq("id", bizDayworkItem.getDayworkId()).list().get(0);
//        if(!turnoverList.isEmpty()) {
//            bizDayworkItemService.removeById(turnoverList.get(0));
//            //修改daywork主表的工段和合格数信息
//            daywork.setDeptId(bizDayworkItem.getDeptId());
//            daywork.setTemporaryProcessQualifiedNum(daywork.getProcessQualifiedNum());
//            bizDayworkService.updateById(daywork);
//        }
//        //判断该条报工是否是该工序最后一条报工
//        List<BizDayworkItem> hasList = bizDayworkItemService.getListByDayworkIds(id).stream().filter(item -> item.getProcessStepNumber().equals(bizDayworkItem.getProcessStepNumber())).collect(Collectors.toList());
//        if(hasList.isEmpty()) {
//            //如果没查到，则代表删除的这一条为最后一条，则新增一条未开始的报工
//            BizDayworkItem dayworkItem = new BizDayworkItem();
//            dayworkItem.setDayworkId(bizDayworkItem.getDayworkId());;
//            if (daywork.getIsAmend() == 1) {
//                dayworkItem.setFormDayworkItemId(1L);
//            }
//            dayworkItem.setProcessStepNumber(bizDayworkItem.getProcessStepNumber());
//            dayworkItem.setProductionPlanDetailId(bizDayworkItem.getProductionPlanDetailId());
//            dayworkItem.setTechnologicalProcessId(bizDayworkItem.getTechnologicalProcessId());
//            dayworkItem.setUserId(bizDayworkItem.getUserId());
//            dayworkItem.setUserName(bizDayworkItem.getUserName());
//            dayworkItem.setProdNum(bizDayworkItem.getProdNum());
//            dayworkItem.setNickName(bizDayworkItem.getNickName());
//            //0114 这里要把前段显示的item的processId传进来
//            dayworkItem.setProcessId(bizDayworkItem.getProcessId());
//            dayworkItem.setTechnologicalProcessDetailId(bizDayworkItem.getTechnologicalProcessDetailId());
//            dayworkItem.setDeptId(bizDayworkItem.getDeptId());
//            dayworkItem.setDeptName(bizDayworkItem.getDeptName());
//            dayworkItem.setLotId(bizDayworkItem.getLotId());
//            bizDayworkItemService.save(dayworkItem);
//        }
        return success();
    }

    /**
     * 获取批次工序信息
     */
    @GetMapping("/getDayworkItemProcess/{id}")
    public AjaxResult getDayworkItemProcess(@PathVariable("id") String lotCode) {
        //查询批次信息
        HashMap<String, Object> map = new HashMap<>();
        try {
            BizLot lot = new BizLot();
            lot.setLotCode(lotCode);
            List<BizLot> lotList = bizLotService.getList(lot);
            lot = lotList.size() > 0 ? lotList.get(0) : null;
            if (lot != null) {
                //查询当前工序
                BizDaywork daywork = new BizDaywork();
                BizDayworkItem item = new BizDayworkItem();
                daywork.setLotId(lot.getId());
                daywork.setProductionPlanDetailId(lot.getProductionPlanDetailId());
                daywork.setLotCode(lotCode);
                //查询报工数据
                List<BizDaywork> dayworkList = bizDayworkService.getList(daywork);
                //没有数据表示未开始报工
                if (dayworkList.size() > 0) {
                    daywork = bizDayworkService.getList(daywork).get(0);
                    item.setDayworkId(daywork.getId());
                    item = bizDayworkItemService.getList(item).get(0);
                    if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
                        BizLotTechnologicalProcessDetail processDetail = bizLotTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
                        item.setProcessStepNumber(processDetail.getProcessStepNumber());
                    } else {
                        BizTechnologicalProcessDetail processDetail = bizTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
                        item.setProcessStepNumber(processDetail.getProcessStepNumber());
                    }
                } else {
                    item = null;
                }
                //是否单批单改
                if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
                    BizLotTechnologicalProcessDetail amendDetail = new BizLotTechnologicalProcessDetail();
                    amendDetail.setLotTechnologicalProcessId(lot.getTechnologicalProcessId());
                    List<BizLotTechnologicalProcessDetail> amendDetailList = bizTechnologicalAmendDetailService.getList(amendDetail);
                    map.put("processDetails", amendDetailList);
                } else {
                    //查询工序列表
                    List<BizTechnologicalProcessDetail> processDetails = bizTechnologicalProcessDetailService.getProcessDetailsByTechnicalProcessId(lot.getTechnologicalProcessId());
                    map.put("processDetails", processDetails);
                }
                map.put("dayworkItem", item);
                map.put("lot", lot);
            } else {
                throw new RuntimeException("未查询到该批次信息！");
            }
        } catch (Exception e) {
            throw new RuntimeException("批次信息异常！");
        }

        return success(map);
    }

    @PostMapping("/checkUseEquipment")
    public AjaxResult checkUseEquipment(@RequestBody List<BizProductionResourceGroupDetail> bizProductionResourceGroupDetailList) {
        List<BizProductionResourceGroupDetail> equipmentDetail = bizProductionResourceGroupDetailList.stream().filter(item ->
                item.getCommonType().equals("equipmentDetail")).collect(Collectors.toList());
        String errorMessage = "";
        for (BizProductionResourceGroupDetail item : equipmentDetail) {
            List<BizDayworkItem> list = bizProductionResourceGroupDetailService.checkUseEquipment(item);
            if (list.size() > 0) {
                errorMessage = errorMessage + item.getCommonName() + ",";
            }
        }
        if ("".equals(errorMessage)) {
            return success();
        }
        return error(errorMessage + "设备正在使用不能删除");
    }

    /**
     * 获取报工记录的工段
     */
    @GetMapping("/getDeptInfo")
    public AjaxResult getDeptList(BizDayworkItem bizDayworkItem) {
        return success(bizDayworkItemService.getDeptInfo(bizDayworkItem));
    }

    /**
     * 获取计划详情
     */
    @GetMapping("/getDayworkInfo")
    public TableDataInfo getDayworkInfo(BizDaywork daywork) {
        return getDataTable(bizDayworkService.getDayworkInfo(daywork));
    }

    @GetMapping("/getProducedLot")
    public TableDataInfo getProducedLot(BizProductionPlanDetail data) {

        return getDataTable(bizDayworkService.getProducedLot(data));
    }

    @GetMapping("/getItemProcess")
    public AjaxResult getItemProcess(BizProductionPlanDetail data) {
        //拿到生产计划
        BizProductionPlanDetail bizProductionPlanDetail = bizProductionPlanDetailService.query().eq("production_plan_No", data.getProductionPlanNo()).eq("line_number", data.getLineNumber()).list().get(0);
        List<BizDaywork> bizDayworkList = bizDayworkService.query().eq("production_plan_detail_id", bizProductionPlanDetail.getId()).list();
        List<Long> dayworkIds = bizDayworkList.stream().map(BizDaywork::getId).collect(Collectors.toList());
        List<BizProcess> newProcess = new ArrayList<>();
        if (!dayworkIds.isEmpty()) {
            List<BizDayworkItem> dayworkItemList = bizDayworkItemService.getListByDayworkIds(dayworkIds);
            //拿到所有的process
            List<BizProcess> processList = bizProcessService.query().list();
            for (BizDaywork item : bizDayworkList) {
                BizProcess process = new BizProcess();
                BizDayworkItem bizDayworkItem = dayworkItemList.stream().filter(item1 -> item1.getDayworkId().equals(item.getId())).sorted(Comparator.comparing(BizDayworkItem::getProcessStepNumber).reversed()).collect(Collectors.toList()).get(0);
                process.setValue(bizDayworkItem.getProcessId());
                process.setLabel(processList.stream().filter(item1 -> item1.getId().equals(process.getValue())).collect(Collectors.toList()).get(0).getProcessAlias());
                if (newProcess.stream().filter(item1 -> item1.getValue().equals(process.getValue())).collect(Collectors.toList()).size() == 0) {
                    newProcess.add(process);
                }
            }
        }
        return success(newProcess);
    }

    /**
     * 获取计划详情的工段列表
     */
    @GetMapping("/getDeptList")
    public AjaxResult getDeptList(BizDaywork daywork) {
        return success(bizDayworkService.getDeptList(daywork));
    }

    /**
     * 获取计划详情的工序列表
     */
    @GetMapping("/getDayworkProcessList")
    public AjaxResult getDayworkProcessList(BizDaywork daywork) {
        return success(bizDayworkService.getDayworkProcessList(daywork));
    }

    /**
     * 导出批次报工列表
     */
    @Log(title = "报工", businessType = BusinessType.EXPORT)
    @PostMapping("/download")
    public void download(HttpServletResponse response, BizDayworkItem bizDayworkItem) throws NoSuchFieldException, IllegalAccessException {
        List<BizDayworkItem> itemList = bizDayworkService.listReportItem(bizDayworkItem);
        List<Long> lotIds = itemList.stream().filter(item -> item.getProdNum() == 0).map(BizDayworkItem::getLotId).collect(Collectors.toList());
        List<BizLot> lotList = bizLotService.query().in("id", lotIds.isEmpty() ? Collections.singletonList(0L) : itemList.stream().filter(item -> item.getProdNum() == 0).map(BizDayworkItem::getLotId).collect(Collectors.toList())).list();
        //首序投产数为0，所以查询如果prodNum为0，其他是第一条报工的时候，给prodNum赋值单批量
        itemList.forEach(item -> {
            if (item.getProdNum() == 0) {
                //拿到第一条报工的工序步骤，为了判断这个投产数为0，是不是该工序步骤的报工，如果是，就给它的投产量赋值为单批量
                String processStepNumber = itemList.stream().filter(v -> v.getDayworkId().equals(item.getDayworkId())).sorted(Comparator.comparing(BizDayworkItem::getProcessStepNumber)).collect(Collectors.toList()).get(0).getProcessStepNumber();
                if (item.getProcessStepNumber().equals(processStepNumber)) {
                    item.setProdNum(lotList.stream().filter(lot -> lot.getId().equals(item.getLotId())).findFirst().get().getProductionQuantity());
                }
            }
            if ("0".equals(item.getStatus())) {
                item.setStatus("未开始");
            }
            if ("1".equals(item.getStatus())) {
                item.setStatus("进行中");
            }
            if ("2".equals(item.getStatus())) {
                item.setStatus("报工结束");
            }
            if ("3".equals(item.getStatus())) {
                item.setStatus("工序已完成");
            }
            if (item.getWorkingHours() != null) {
                int seconds = (int) Math.floor((item.getWorkingHours() / 1000) % 60);
                int minutes = (int) Math.floor((item.getWorkingHours() / (1000 * 60)) % 60);
                int hours = (int) Math.floor(item.getWorkingHours() / (1000 * 60 * 60));
                String time = hours + "小时" + minutes + "分钟" + seconds + "秒";
                item.setTotalTime(time);
            }
            if (item.getRejectNum() == null) {
                item.setRejectNum(0);
            }
        });
        ExcelUtil<BizDayworkItem> util = new ExcelUtil<>(BizDayworkItem.class);
        util.exportExcel(response, itemList, "报工数据");
    }

    /**
     * 导出批次报工列表
     */
    @Log(title = "报工", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizDaywork bizDaywork) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizDaywork);
        List<BizDaywork> list = bizDayworkService.getList(bizDaywork);
        ExcelUtil<BizDaywork> util = new ExcelUtil<>(BizDaywork.class);
        util.exportExcel(response, list, "报工数据");
    }

    /**
     * 获取报工详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(bizDayworkService.getById(id));
    }

    /**
     * 获取报工废品数信息
     */
    @GetMapping("/getDayworkItemRejectList")
    public AjaxResult getDayworkItemRejectList(BizDayworkItemReject bizDayworkItemReject) {
        return success(bizDayworkItemRejectService.getList(bizDayworkItemReject));
    }

    /**
     * 查询当前是否为首序
     */
    @Log(title = "是否为首序拆线呢", businessType = BusinessType.SELECT)
    @GetMapping(value = "/getIsFirstOrder/{dayWorkId}")
    public AjaxResult getIsFirstOrder(@PathVariable("dayWorkId") Long dayWorkId) {
        HashMap<String, Object> map = new HashMap<>();
        //根据dayWorkId查询出当前工序的报工记录(按照工序筛选每道工序只查询一条数据)
        List<BizDayworkItem> list = bizDayworkItemService.getItemScreen(dayWorkId);
        //查询出来的数据等于1条则为首序
        if (list.size() == 1) {
            map.put("isFirstOrder", true);
        } else {
            map.put("isFirstOrder", false);
        }
        return success(map);
    }

    /**
     * 修改dayworkItem和dayworkItemReject
     *
     * @param bizDayworkItem
     * @return TableDataInfo
     */
    @Transactional
    @PostMapping("/updateDayworkItemInfo")
    public AjaxResult updateDayworkItemInfo(@RequestBody BizDayworkItem bizDayworkItem) {
//        String tenantId = request.getHeader("tenantId");
//        bizDayworkItem.setTenantId(Long.valueOf(tenantId));
        if (bizDayworkItem.getStatus().equals("3")) {
            //往合格数中间表新增数据
            BizDayworkQualified bizDayworkQualified = new BizDayworkQualified();
            bizDayworkQualified.setDayworkId(bizDayworkItem.getDayworkId());
            bizDayworkQualified.setTemporaryProcessQualifiedNum(bizDayworkItem.getTemporaryProcessQualifiedNum());
            bizDayworkQualified.setProcessId(bizDayworkItem.getProcessId());
            bizDayworkQualified.setProcessQualifiedNum(bizDayworkItem.getProdNum());
            bizDayworkQualified.setProductionPlanDetailId(bizDayworkItem.getProductionPlanDetailId());
            bizDayworkQualifiedService.save(bizDayworkQualified);
        }
        //前端判断，如果状态从工序已完成变成报工结束时，将中间表数据删除
        if (bizDayworkItem.getFlag() != null) {
            QueryWrapper<BizDayworkQualified> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("deleted", 0);
            queryWrapper.eq("daywork_id", bizDayworkItem.getDayworkId());
            queryWrapper.eq("process_id", bizDayworkItem.getProcessId());
            List<BizDayworkQualified> list = bizDayworkQualifiedService.list(queryWrapper);
            if (list.size() > 0) {
                bizDayworkQualifiedService.removeById(list.get(0));
            }

        }
        //修改biz_daywork表的process_qualified_num(这道序的合格总数）
        QueryWrapper<BizDaywork> query = new QueryWrapper<>();
        query.eq("deleted", 0);
        query.eq("id", bizDayworkItem.getDayworkId());
        BizDaywork daywork = bizDayworkService.list(query).get(0);
        daywork.setTemporaryProcessQualifiedNum(bizDayworkItem.getTemporaryProcessQualifiedNum());
        bizDayworkService.updateById(daywork);
        //修改报工信息
        bizDayworkItemService.updateDayworkItem((bizDayworkItem));
        //修改报工信息的废弃表
        QueryWrapper<BizDayworkItemReject> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("daywork_item_id", bizDayworkItem.getId());
        queryWrapper.eq("deleted", 0);
        List<BizDayworkItemReject> oldList = bizDayworkItemRejectService.list(queryWrapper);
        saveOrUpdateBatch(bizDayworkItemRejectService, bizDayworkItem.getGroupRejectList(), oldList);
        return success();
    }

    @GetMapping("/processesForOutsourceByFirst")
    public TableDataInfo listProcessesForOutsourceByFirst(BizLot bizLot) {
        //查找当前生产子计划的批次报工信息，找到最后一条包报工信息
        //List<BizDayworkItem> dayworkItemList = bizDayworkItemService.getList(dayworkItem);
        //用户再次提交时传入外协明细id
        BizLot currentLot = bizLotService.getById(bizLot.getId());
        currentLot.setKeyword(bizLot.getKeyword());
        //根据当前工序查找之后的工序列表
        //是否单批单改或废品回用
//        startPage();
        if (currentLot.getIsAmend() == 1 || currentLot.getIsWasteRecycling() == 1) {
            List<BizLotTechnologicalProcessDetail> processList = bizLotTechnologicalProcessDetailService.getProcessList(currentLot);
            return getDataTable(processList);
        } else {
            List<BizTechnologicalProcessDetail> processList = bizTechnologicalProcessDetailService.getProcessList(currentLot);
            return getDataTable(processList);
        }
    }
}
