package cn.ezhizao.project.business.controller;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.domain.BizDayworkItem;
import cn.ezhizao.project.business.domain.BizDayworkItemReject;
import cn.ezhizao.project.business.domain.BizLot;
import cn.ezhizao.project.business.domain.BizOutsourcedOrderDetail;
import cn.ezhizao.project.business.service.IBizDayworkItemRejectService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


/**
 * 报工废品记录Controller
 *
 * @author ezhizao
 * @date 2024-06-18
 */
@RestController
@RequestMapping("/business/reject")
public class BizDayworkItemRejectController extends BaseController
{
    @Resource
    private IBizDayworkItemRejectService bizDayworkItemRejectService;

    /**
     * 查询报工废品记录列表
     */
    @GetMapping("/list")
    public TableDataInfo list(BizDayworkItemReject bizDayworkItemReject) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizDayworkItemReject);
        startPage();
        List<BizDayworkItemReject> list = bizDayworkItemRejectService.getList(bizDayworkItemReject);
        return getDataTable(list);
    }
    @GetMapping("/rejectReportList")
    public AjaxResult rejectReportList(BizDayworkItemReject bizDayworkItemReject) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizDayworkItemReject);
        List<BizDayworkItemReject> list = bizDayworkItemRejectService.getRejectReport(bizDayworkItemReject);
        long count = list.stream().mapToInt(BizDayworkItemReject::getTotalRejectNum).sum();
        return success(count);
    }
    @GetMapping("/rejectReport")
    public TableDataInfo getRejectReport(BizDayworkItemReject bizDayworkItemReject) throws NoSuchFieldException, IllegalAccessException
    {
        startPage();
        List<BizDayworkItemReject> list = bizDayworkItemRejectService.getRejectReport(bizDayworkItemReject);
        return getDataTable(list);
    }
    @GetMapping("/rejectRecords")
    public TableDataInfo getRejectRecords(BizDayworkItemReject bizDayworkItemReject) throws NoSuchFieldException, IllegalAccessException
    {
        List<BizDayworkItemReject> list = bizDayworkItemRejectService.getRejectRecords(bizDayworkItemReject);
        return getDataTable(list);
    }
    @PostMapping("/download")
    public void download(HttpServletResponse response, BizDayworkItemReject bizDayworkItemReject) throws NoSuchFieldException, IllegalAccessException {
        List<BizDayworkItemReject> list = bizDayworkItemRejectService.getRejectReport(bizDayworkItemReject);
        ExcelUtil<BizDayworkItemReject> util = new ExcelUtil<>(BizDayworkItemReject.class);
        util.exportExcel(response, list, "废品报表数据");
    }

    /**
     * 导出报工废品记录列表
     */
    @PreAuthorize("@ss.hasPermi('business:reject:export')")
    @Log(title = "报工废品记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizDayworkItemReject bizDayworkItemReject) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizDayworkItemReject);
        List<BizDayworkItemReject> list = bizDayworkItemRejectService.getList(bizDayworkItemReject);
        ExcelUtil<BizDayworkItemReject> util = new ExcelUtil<BizDayworkItemReject>(BizDayworkItemReject.class);
        util.exportExcel(response, list, "报工废品记录数据");
    }

    /**
     * 获取报工废品记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('business:reject:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizDayworkItemRejectService.getById(id));
    }

    /**
     * 新增报工废品记录
     */
    @Log(title = "报工废品记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizDayworkItemReject bizDayworkItemReject) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizDayworkItemReject);
        return toAjax(bizDayworkItemRejectService.save(bizDayworkItemReject));
    }

    /**
     * 修改报工废品记录
     */
    @Log(title = "报工废品记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizDayworkItemReject bizDayworkItemReject) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizDayworkItemReject);
        return toAjax(bizDayworkItemRejectService.updateById(bizDayworkItemReject));
    }

    /**
     * 删除报工废品记录
     */
    @Log(title = "报工废品记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(bizDayworkItemRejectService.removeBatchByIds(ids));
    }
}
