package cn.ezhizao.project.business.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.domain.BizDayworkItemReject;
import cn.ezhizao.project.business.domain.BizProcessInspecion;
import cn.ezhizao.project.business.service.IBizDayworkItemConsultService;
import cn.ezhizao.project.business.service.IBizDayworkItemRejectService;
import cn.ezhizao.project.business.service.IBizProcessInspecionService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


/**
 * 序检主表Controller
 *
 * @author ezhizao
 * @date 2024-06-27
 */
@RestController
@RequestMapping("/business/inspecion")
public class BizProcessInspecionController extends BaseController
{
    @Resource
    private IBizProcessInspecionService bizProcessInspecionService;
    @Resource
    private IBizDayworkItemRejectService bizDayworkItemRejectService;
    @Resource
    private IBizDayworkItemConsultService bizDayworkItemConsultService;

    /**
     * 查询序检主表列表
     */
    @GetMapping("/list")
    public TableDataInfo list(BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizProcessInspecion);
        startPage();
        List<BizProcessInspecion> list = bizProcessInspecionService.getList(bizProcessInspecion);
        return getDataTable(list);
    }
    @GetMapping("/listInfo")
    public TableDataInfo listInfo(BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException
    {
        startPage();
        List<BizProcessInspecion> list = bizProcessInspecionService.getInfo(bizProcessInspecion);
        return getDataTable(list);
    }

    /**
     * 导出序检主表列表
     */
    @Log(title = "序检主表", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizProcessInspecion);
        List<BizProcessInspecion> list = bizProcessInspecionService.getList(bizProcessInspecion);
        ExcelUtil<BizProcessInspecion> util = new ExcelUtil<BizProcessInspecion>(BizProcessInspecion.class);
        util.exportExcel(response, list, "序检主表数据");
    }

    /**
     * 获取序检主表详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        BizProcessInspecion bizProcessInspecion = bizProcessInspecionService.getProcessInspectionById(id).get(0);
       bizProcessInspecion.setRejectList(bizDayworkItemRejectService.query().eq("process_inspection_id", id).list());
       bizProcessInspecion.setConsultList(bizDayworkItemConsultService.query().eq("process_inspection_id", id).list());
        return success(bizProcessInspecion);
    }

    /**
     * 新增序检主表
     */
    @PreAuthorize("@ss.hasPermi('business:inspecion:add')")
    @Log(title = "序检主表", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizProcessInspecion);
        return toAjax(bizProcessInspecionService.save(bizProcessInspecion));
    }

    /**
     * 修改序检主表
     */
    @PreAuthorize("@ss.hasPermi('business:inspecion:edit')")
    @Log(title = "序检主表", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizProcessInspecion);
        return toAjax(bizProcessInspecionService.updateById(bizProcessInspecion));
    }

    /**
     * 删除序检主表
     */
    @PreAuthorize("@ss.hasPermi('business:inspecion:remove')")
    @Log(title = "序检主表", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(bizProcessInspecionService.removeBatchByIds(ids));
    }
}
