package cn.ezhizao.project.business.service.impl;

import cn.ezhizao.project.business.domain.*;
import cn.ezhizao.project.business.mapper.BizDayworkItemMapper;
import cn.ezhizao.project.business.mapper.BizDayworkMapper;
import cn.ezhizao.project.business.service.IBizDayworkItemService;
import cn.ezhizao.project.system.domain.SysDept;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 报工记录Service业务层处理
 *
 * @author ezhizao
 * @date 2023-12-04
 */
@Service
public class BizDayworkItemServiceImpl extends ServiceImpl<BizDayworkItemMapper, BizDayworkItem> implements IBizDayworkItemService {
    @Resource
    private BizDayworkItemMapper bizDayworkItemMapper;
    @Resource
    private BizDayworkMapper bizDayworkMapper;
    @Resource
    private BizOutsourcedOrderDetailProcessServiceImpl bizOutsourcedOrderDetailProcessService;
    @Resource
    private BizOutsourcedOrderServiceImpl bizOutsourcedOrderService;
    @Resource
    private BizOutsourcedOrderDetailServiceImpl bizOutsourcedOrderDetailService;

    /**
     * 查询报工记录列表
     *
     * @param bizDayworkItem 报工记录
     * @return 报工记录
     */
    @Override
    public List<BizDayworkItem> getList(BizDayworkItem bizDayworkItem) {
        return bizDayworkItemMapper.getList(bizDayworkItem);
    }

    @Override
    public List<BizDayworkItem> getOutsourcesItemList(BizDayworkItem bizDayworkItem) {
        return bizDayworkItemMapper.getOutsourcesItemList(bizDayworkItem);
    }

    @Override
    public List<SysDept> getDeptInfo(BizDayworkItem bizDayworkItem) {
        return bizDayworkItemMapper.getDeptInfo(bizDayworkItem);
    }

    @Override
    public List<BizDayworkItem> getListByDayworkIds(List<Long> ids) {
        return bizDayworkItemMapper.getListByDayworkIds(ids);
    }

    @Override
    public List<BizDayworkItem> getListByDayworkIdsForOutsourced(List<Long> ids) {
        return bizDayworkItemMapper.getListByDayworkIdsForOutsourced(ids);
    }

    /**
     * 查询设备信息
     *
     * @param bizProductionResourceGroupDetail
     * @return
     */
    @Override
    public BizProductionResourceGroupDetail getEquipmentInfo(BizProductionResourceGroupDetail bizProductionResourceGroupDetail) {
        return bizDayworkItemMapper.getEquipmentInfo(bizProductionResourceGroupDetail);
    }

    /**
     * 物理删除
     *
     * @param bizDayworkItem
     * @return 删除结果
     */
    @Override
    public int physicalDelete(BizDayworkItem bizDayworkItem) {
        return bizDayworkItemMapper.physicalDelete(bizDayworkItem);
    }

    @Override
    public void updateDayworkItem(BizDayworkItem bizDayworkItem) {
        bizDayworkItemMapper.updateDayworkItem(bizDayworkItem);
    }

    @Override
    public List<BizDayworkItem> getItemScreen(Long dayWorkId) {
        return bizDayworkItemMapper.getItemScreen(dayWorkId);
    }

    @Override
    public List<BizDayworkItem> selectList(BizDayworkItem item) {
        return bizDayworkItemMapper.selectList(item);
    }

    @Override
    public int updateTechnologicalProcessDetailId(Long dayWorkId) {
        return bizDayworkItemMapper.updateTechnologicalProcessDetailId(dayWorkId);
    }

    @Override
    public List<BizDayworkItem> selectLatestItemByDayworkIds(List<Long> dayworkIds) {
        return bizDayworkItemMapper.selectLatestItemByDayworkIds(dayworkIds);
    }

    @Override
    public List<BizDayworkItem> selectItemByOutsourcedDetailIds(List<Long> ids) {
        return bizDayworkItemMapper.selectItemByOutsourcedDetailIds(ids);
    }

    @Override
    @Transactional
    public void saveDayworkItemForOutsource(List<BizOutsourcedOrderDetail> outsourcedOrderDetails) {
        /**********查询周转新信息修改周转状态，新增一条外协部门报工信息，将对应daywork状态修改为进行中************/
        List<BizDayworkItem> dayworkItems = new ArrayList<>();//原周转信息集合
        List<BizDayworkItem> newDayworkItems = new ArrayList<>();//新增报工信息集合
        //修改报工信息
        outsourcedOrderDetails.forEach(detail -> {
            //查询周转信息
            BizDayworkItem bizDayworkItem = new BizDayworkItem();
            bizDayworkItem.setDayworkId(detail.getDayworkId());
            bizDayworkItem.setOutsourceDetailId(detail.getId());
            bizDayworkItem = bizDayworkMapper.getItemListFromOutsourced(bizDayworkItem).get(0);
            //修改状态
            bizDayworkItem.setStatus("7");
            dayworkItems.add(bizDayworkItem);
            //每一道工序新增一条报工信息
            BizDayworkItem finalBizDayworkItem = bizDayworkItem;
            detail.getProcesses().forEach(process -> {
                //新的报工信息
                BizDayworkItem newBizDayworkItem = new BizDayworkItem();
                newBizDayworkItem.setDayworkId(detail.getDayworkId());
                newBizDayworkItem.setOutsourceDetailId(detail.getId());
                newBizDayworkItem.setLotId(detail.getLotId());
                newBizDayworkItem.setProcessId(process.getProcessId());
                newBizDayworkItem.setProcessStepNumber(process.getProcessStepNumber());
                newBizDayworkItem.setStatus("1");
                newBizDayworkItem.setProdNum(detail.getProductNum());
                newBizDayworkItem.setDeptId(finalBizDayworkItem.getDeptId());
                newBizDayworkItem.setDeptName(finalBizDayworkItem.getDeptName());
                newBizDayworkItem.setStartTime(new Date());
                newBizDayworkItem.setDeptCode(finalBizDayworkItem.getDeptCode());
                newBizDayworkItem.setProductionPlanDetailId(finalBizDayworkItem.getProductionPlanDetailId());
                newBizDayworkItem.setTechnologicalProcessId(finalBizDayworkItem.getTechnologicalProcessId());
                newBizDayworkItem.setTechnologicalProcessDetailId(process.getTechnologicalProcessDetailId());
                newBizDayworkItem.setIsWx(1);
                newDayworkItems.add(newBizDayworkItem);
            });
        });
        saveOrUpdateBatch(dayworkItems);
        saveBatch(newDayworkItems);
    }

    @Override
    @Transactional
    public void updateDayworkItemForOutsource(List<BizOutsourcedOrderDetail> outsourcedOrderDetails,List<BizOutsourcedOrderDetail> oldOutsourcedOrderDetails) {
        //编辑发出单对dayworkItem进行操作
        List<Long> dayworkIds = outsourcedOrderDetails.stream().filter(item ->item.getProductionDeptId() ==null || item.getProductionDeptId() ==0 ).map(BizOutsourcedOrderDetail::getDayworkId).collect(Collectors.toList());
        List<Long> outsourcedDetailIds = outsourcedOrderDetails.stream().filter(item ->item.getProductionDeptId() ==null || item.getProductionDeptId() ==0).map(BizOutsourcedOrderDetail::getId).collect(Collectors.toList());
        List<Long> totalIds = new ArrayList<>();
        totalIds.addAll(dayworkIds);
        totalIds.addAll(oldOutsourcedOrderDetails.stream().filter(item ->item.getProductionDeptId() ==null || item.getProductionDeptId() ==0).map(BizOutsourcedOrderDetail::getDayworkId).collect(Collectors.toList()));
        List<BizDayworkItem> dayworkItemList = getListByDayworkIds(totalIds.stream().distinct().collect(Collectors.toList()));
        //如果是新增的外协明细，则拿到报工信息最新一条，如果is_wx = 0，则吧状态置为7
        List<BizDayworkItem> needUpdateStatus = new ArrayList<>();
        for (int i = 0; i < dayworkIds.size(); i++) {
            Long dayworkId = dayworkIds.get(i);
            BizDayworkItem bizDayworkItem = dayworkItemList.stream().filter(item -> item.getDayworkId().equals(dayworkId)).sorted(Comparator.comparing(BizDayworkItem::getCreateTime).reversed()).collect(Collectors.toList()).get(0);
            if (bizDayworkItem.getIsWx() == 0) {
                bizDayworkItem.setStatus("7");
                needUpdateStatus.add(bizDayworkItem);
            }
        }
        if (needUpdateStatus.size() > 0) {
            updateBatchById(needUpdateStatus);
        }
        //判断dayworkItem需要删除、新增、修改
        List<BizDayworkItem> needDeletedDayworkItem = new ArrayList<>();
        List<BizDayworkItem> needAddedDayworkItem = new ArrayList<>();
        List<BizDayworkItem> needUpdateDayworkItem = new ArrayList<>();
        //通过明细id查到添加的dayworkItem
        //通过外协发出单主表id查出原发出单的外协明细（页面删除外协明细）
        List<Long> originalOutsourcedDetailIds = oldOutsourcedOrderDetails.stream().filter(item ->item.getProductionDeptId() ==null || item.getProductionDeptId() ==0).map(BizOutsourcedOrderDetail::getId).collect(Collectors.toList());
        originalOutsourcedDetailIds.forEach(item ->{
            //如果新的发出单明细没有不包含旧的，则查询出来dayworkItem然后删除
            if (!outsourcedDetailIds.contains(item)){
                needDeletedDayworkItem.addAll(dayworkItemList.stream().filter(info ->info.getOutsourceDetailId().equals(item)).collect(Collectors.toList()));
            }
        });
        outsourcedOrderDetails.stream().filter(item ->item.getProductionDeptId() ==null || item.getProductionDeptId() ==0).forEach(detail -> {
            List<BizDayworkItem> originalDayworkItem = dayworkItemList.stream().filter(item -> item.getOutsourceDetailId().equals(detail.getId())).collect(Collectors.toList());
            List<String> newProcessStepNumberList = detail.getProcesses().stream().map(BizOutsourcedOrderDetailProcess::getProcessStepNumber).collect(Collectors.toList());
            //拿到最新的报工明细的工段
            BizDayworkItem lastDayworkItem = dayworkItemList.stream().filter(item -> item.getDayworkId().equals(detail.getDayworkId())).sorted(Comparator.comparing(BizDayworkItem::getCreateTime).reversed()).collect(Collectors.toList()).get(0);
            if(!originalDayworkItem.isEmpty()) {
                //需要删除的报工明细
                List<String> oldProcessStepNumberList = originalDayworkItem.stream().map(BizDayworkItem::getProcessStepNumber).collect(Collectors.toList());
                originalDayworkItem.forEach(item ->{
                        if(!newProcessStepNumberList.contains(item.getProcessStepNumber())) {
                            needDeletedDayworkItem.add(item);
                        }
                    });
                //需要新增的报工明细
                detail.getProcesses().forEach(process -> {
                    //新的报工信息
                    if(!oldProcessStepNumberList.contains(process.getProcessStepNumber())) {
                        BizDayworkItem newBizDayworkItem = new BizDayworkItem();
                        newBizDayworkItem.setDayworkId(detail.getDayworkId());
                        newBizDayworkItem.setOutsourceDetailId(detail.getId());
                        newBizDayworkItem.setLotId(detail.getLotId());
                        newBizDayworkItem.setProcessId(process.getProcessId());
                        newBizDayworkItem.setProcessStepNumber(process.getProcessStepNumber());
                        newBizDayworkItem.setStatus("1");
                        newBizDayworkItem.setProdNum(detail.getProductNum());
                        newBizDayworkItem.setDeptId(lastDayworkItem.getDeptId());
                        newBizDayworkItem.setDeptName(lastDayworkItem.getDeptName());
                        newBizDayworkItem.setStartTime(new Date());
                        newBizDayworkItem.setProductionPlanDetailId(lastDayworkItem.getProductionPlanDetailId());
                        newBizDayworkItem.setTechnologicalProcessId(lastDayworkItem.getTechnologicalProcessId());
                        newBizDayworkItem.setTechnologicalProcessDetailId(process.getTechnologicalProcessDetailId());
                        newBizDayworkItem.setIsWx(1);
                        needAddedDayworkItem.add(newBizDayworkItem);
                    }else{
                        //编辑报工信息
                        BizDayworkItem newBizDayworkItem = new BizDayworkItem();
                        newBizDayworkItem.setDayworkId(detail.getDayworkId());
                        newBizDayworkItem.setOutsourceDetailId(detail.getId());
                        newBizDayworkItem.setLotId(detail.getLotId());
                        newBizDayworkItem.setProcessId(process.getProcessId());
                        newBizDayworkItem.setProcessStepNumber(process.getProcessStepNumber());
                        newBizDayworkItem.setStatus("1");
                        newBizDayworkItem.setProdNum(detail.getProductNum());
                        newBizDayworkItem.setDeptId(lastDayworkItem.getDeptId());
                        newBizDayworkItem.setDeptName(lastDayworkItem.getDeptName());
//                        newBizDayworkItem.setStartTime(new Date());
                        newBizDayworkItem.setProductionPlanDetailId(lastDayworkItem.getProductionPlanDetailId());
                        newBizDayworkItem.setTechnologicalProcessId(lastDayworkItem.getTechnologicalProcessId());
                        newBizDayworkItem.setTechnologicalProcessDetailId(process.getTechnologicalProcessDetailId());
                        newBizDayworkItem.setIsWx(1);
                        needUpdateDayworkItem.add(newBizDayworkItem);
                    }
                });
                }else{
                //执行新增
                detail.getProcesses().forEach(process -> {
                    //新的报工信息
                    BizDayworkItem newBizDayworkItem = new BizDayworkItem();
                    newBizDayworkItem.setDayworkId(detail.getDayworkId());
                    newBizDayworkItem.setOutsourceDetailId(detail.getId());
                    newBizDayworkItem.setLotId(detail.getLotId());
                    newBizDayworkItem.setProcessId(process.getProcessId());
                    newBizDayworkItem.setProcessStepNumber(process.getProcessStepNumber());
                    newBizDayworkItem.setStatus("1");
                    newBizDayworkItem.setProdNum(detail.getProductNum());
                    newBizDayworkItem.setDeptId(lastDayworkItem.getDeptId());
                    newBizDayworkItem.setDeptName(lastDayworkItem.getDeptName());
                    newBizDayworkItem.setStartTime(new Date());
                    newBizDayworkItem.setProductionPlanDetailId(lastDayworkItem.getProductionPlanDetailId());
                    newBizDayworkItem.setTechnologicalProcessId(lastDayworkItem.getTechnologicalProcessId());
                    newBizDayworkItem.setTechnologicalProcessDetailId(process.getTechnologicalProcessDetailId());
                    newBizDayworkItem.setIsWx(1);
                    needAddedDayworkItem.add(newBizDayworkItem);
                });
            }
        });
        if(!needDeletedDayworkItem.isEmpty()) {
            removeBatchByIds(needDeletedDayworkItem);
        }
        if(!needAddedDayworkItem.isEmpty()){
            saveBatch(needAddedDayworkItem);
        }
        if(!needUpdateDayworkItem.isEmpty()) {
            updateBatchById(needUpdateDayworkItem);
        }
    }

    @Override
    public void removeByOutsourceIds(List<Long> ids) {
        List<BizDayworkItem> dayworkItemList = selectItemByOutsourcedDetailIds(ids);
        removeBatchByIds(dayworkItemList);

    }

    @Override
    public Long getLatestDayworkItemId(BizDayworkItem bizDayworkItem) {
        return bizDayworkItemMapper.getLatestDayworkItemId(bizDayworkItem);
    }

    @Override
    public BizDayworkItem getLastItem(BizDayworkItem bizDayworkItem) {
        return bizDayworkItemMapper.getLastItem(bizDayworkItem);
    }

    @Override
    public List<BizDayworkItem> getLastFinished(BizDayworkItem dayworkItem) {
        return bizDayworkItemMapper.getLastFinished(dayworkItem);
    }

}
